package handler

import (
	"context"

	"google.golang.org/grpc/metadata"

	"git.loafle.net/commons_go/logging"
	"git.loafle.net/overflow/overflow_gateway_probe/grpc"
	"git.loafle.net/overflow/overflow_gateway_probe/server"
	ogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/protocol/jsonrpc"
	"go.uber.org/zap"
)

func newAuthJSONRpcHandler(ctx context.Context) ogw.ProtocolHandler {
	h := &authJSONRpcHandlers{
		ctx: ctx,
	}

	p := jsonrpc.New(ctx, h)

	return p
}

type authJSONRpcHandlers struct {
	jsonrpc.JSONRpcHandlers
	ctx context.Context
}

func (h *authJSONRpcHandlers) OnRequest(soc ogw.Socket, method string, params []string) (interface{}, error) {
	logging.Logger.Info("OnRequest",
		zap.String("path", soc.Path()),
		zap.String("method", method),
		zap.Any("params", params),
	)

	uid := server.GetUID(soc.Path(), soc)

	md := metadata.Pairs("email", uid)
	ctx := metadata.NewOutgoingContext(context.Background(), md)

	r, err := grpc.Exec(ctx, method, params)
	if err != nil {
		return nil, err
	}

	return r, nil
}

func (h *authJSONRpcHandlers) OnNotify(soc ogw.Socket, method string, params []string) error {

	return nil
}
